#! /usr/bin/python3 -s

import time
import sys
if sys.version_info < (3, 0):
    import httplib
else:
    import http.client as httplib
    StandardError = Exception

# Mendeley HTTP Client

# A client for communicating with the HTTP/JSON Mendeley Desktop Word
# processor API

# simplejson is json 
# simplejson is json (using a generic 'except' and not 'except ImportError'
# because MD-19770. See https://bugs.launchpad.net/ubuntu/+source/libreoffice/+bug/1222823
try: import simplejson as json
except: import json

# For communicating with the Mendeley Desktop HTTP API
class MendeleyHttpClient():
    HOST = "127.0.0.1" # much faster than "localhost" on Windows
                       # see http://cubicspot.blogspot.com/2010/07/fixing-slow-apache-on-localhost-under.html
    PORT = "50002"
    CONTENT_TYPE = "application/vnd.mendeley.wordProcessorApi+json; version=1.0"
    lastRequestTime = -1

    def __init__(self):
        self.previousResponse = self.Response(200, None, None, None)

    class Response:
        def __init__(self, status, contentType, body, request):
            self.status = status
            self.body = body
            self.contentType = contentType
            self.request = request

    class UnexpectedResponse(StandardError):
        def __init__(self, response):

            try:
                message = "response: ", json.dumps(response)
            except:
                message = "status: " + str(response.status)
                try:
                    # not sure why this works but the above doesn't
                    message += ", body: " + json.dumps(response.body.__dict__)
                except:
                    message += ", body: " + str(response.body)
            StandardError.__init__(self, message)

    # Currently this uses the same version number for all API routes,
    # this could be altered to be more fine-grained 
    class Request(object):    
        def __init__(self, verb, path, contentType, acceptType, body):
            self._verb = verb
            self._path = path
            self._body = body  # python dictionary
            self._contentType = contentType
            self._acceptType = acceptType
    
        def verb(self):
            return self._verb
    
        def path(self):
            return self._path
    
        def acceptType(self):
            return self._acceptType

        def contentType(self):
            return self._contentType

        def body(self):
            return json.dumps(self._body)

    class GetRequest(Request):
        def __init__(self, path):
            super(MendeleyHttpClient.GetRequest, self).__init__(
                "GET",
                path,
                "",
                MendeleyHttpClient.CONTENT_TYPE,
                "")
            
    class PostRequest(Request):
        def __init__(self, path, body):
            super(MendeleyHttpClient.PostRequest, self).__init__(
                "POST",
                path,
                MendeleyHttpClient.CONTENT_TYPE,
                MendeleyHttpClient.CONTENT_TYPE,
                body)

    def formattedCitationsAndBibliography_Interactive(self, citationStyleUrl, citationClusters):
        request = self.PostRequest(
            "/formattedCitationsAndBibliography/interactive",
            {
                "citationStyleUrl": citationStyleUrl,
                "citationClusters": citationClusters
            }
            )
        return self.request(request)
    
    def citation_choose_interactive(self, citationEditorHint):
        request = self.PostRequest(
            "/citation/choose/interactive",
            citationEditorHint
            )
        return self.request(request)

    def citation_edit_interactive(self, citationCluster):
        request = self.PostRequest(
            "/citation/edit/interactive",
            citationCluster
            )
        return self.request(request)

    def citation_update_interactive(self, formattedCitationCluster):
        request = self.PostRequest(
            "/citation/update/interactive",
            formattedCitationCluster
            )
        return self.request(request)

    def citationStyle_choose_interactive(self, currentStyleUrl):
        request = self.PostRequest(
            "/citationStyle/choose/interactive",
            currentStyleUrl
            )
        return self.request(request)

    def citationStyle_selected(self):
        request = self.GetRequest(
            "/citationStyle/selected"
            )
        return self.request(request)

    def styleName_getFromUrl(self, styleUrl):
        request = self.PostRequest(
            "/citationStyle/getNameFromUrl",
            styleUrl
            )
        return self.request(request)

    def citationStyles_default(self):
        request = self.GetRequest(
            "/citationStyles/default",
            )
        return self.request(request)

    def citations_merge(self, citationClusters):
        request = self.PostRequest(
            "/citations/merge",
            citationClusters
            )
        return self.request(request)

    def citation_undoManualFormat(self, citationCluster):
        request = self.PostRequest(
            "/citation/undoManualFormat",
            citationCluster
            )
        return self.request(request)

    def wordProcessor_set(self, wordProcessor):
        request = self.PostRequest(
            "/wordProcessor/set",
            wordProcessor
            )
        return self.request(request)

    def testMethods_citationCluster_getFromUuid(self, uuid):
        request = self.PostRequest(
            "/testMethods/citationCluster/getFromUuid",
            uuid
            )
        return self.request(request)
        
    def userAccount(self):
        request = self.GetRequest(
            "/userAccount"
            )
        return self.request(request)

    def citationStylePresentationType(self):
        request = self.GetRequest(
            "/citationStyle/getCitationStylePresentationType"
            )
        return self.request(request)

    def userUuid(self):
        request = self.GetRequest(
            "/userUuid"
            )
        return self.request(request)

    def mendeleyDesktopInfo(self):
        request = self.GetRequest(
            "/mendeleyDesktopInfo"
            )
        return self.request(request)

    def desktopSelectedCitationStyle(self):
        request = self.GetRequest(
            "/"
            )
        return self.request(request)

    # Need to define a class for this.
    # I tried using a object() instance but it doesn't contain a __dict__
    class ResponseBody:
        pass

    # Sets up a connection to Mendeley Desktop, makes a HTTP request and
    # returns the data
    def request(self, requestData):
        headers = {}
        # putting an empty string in Content-Type causes the Mendeley Desktop HTTP
        # server to put the Accept header value in a field called "content-typeaccept"
        # TODO: check where this error comes from
        if requestData.contentType() != "":
            headers["Content-Type"] = requestData.contentType()
        if requestData.acceptType() != "":
            headers["Accept"] = requestData.acceptType()
        startTime = time.time()
        connection = httplib.HTTPConnection(self.HOST + ":" + self.PORT)
        connection.request(requestData.verb(), requestData.path(), requestData.body(), headers)
        response = connection.getresponse()
        data = response.read()
        data = data.decode('utf-8')

        if (response.status == 200 and (not response.getheader("Content-Type") is None) and
                response.getheader("Content-Type") != requestData.acceptType()):
            # TODO: abort if the wrong content type is returned
            pass

        responseBody = MendeleyHttpClient.ResponseBody()
        try:
            responseBody.__dict__.update(json.loads(data))
        except:
            responseBody = data
        connection.close()
        self.lastRequestTime = 1000 * (time.time() - startTime)

        self.previousResponse = \
                self.Response(response.status, response.getheader("Content-Type"), responseBody,
                        requestData)

        return self.previousResponse
#!/usr/bin/python

# Mendeley Desktop API

# This provides a wrapper for OpenOffice basic to use the
# HTTP/JSON Mendeley Desktop Word Processor API.
# It is responsible for building the python dictionaries which represent
# the JSON requests and providing accessor functions for the server responses.

# To run tests, open MendeleyDesktop and run this script
# (it knows that it's in test mode because "import unohelper"
#  will fail when run outside of the OpenOffice.org environment)

# simplejson is json (using a generic 'except' and not 'except ImportError'
# because MD-19770. See: https://bugs.launchpad.net/ubuntu/+source/libreoffice/+bug/1222823
try: import simplejson as json
except: import json

import os
import re
import codecs
import sys
if sys.version_info < (3, 0):
    python3 = False
else:
    python3 = True

# All strings are unicode in Python 3.
# See http://stackoverflow.com/questions/6812031/how-to-make-unicode-string-with-python3
if python3:
    unicode = str

# if DO_NOT_USE_UNO_HELPER environment variable exists:
# it doesn't try to use the unohelper package. Mendeley tests sets this
# variable when needed.
fakeUnoHelper = True
if not 'DO_NOT_USE_UNO_HELPER' in os.environ:
    try:
        import unohelper
        from com.sun.star.task import XJob
        fakeUnoHelper = False
    except ImportError:
        pass

if fakeUnoHelper:
    from MendeleyHttpClient import MendeleyHttpClient

    class unohelper():
        def __init__(self, ctx):
            pass

        class Base():
            def __init__(self, ctx):
                pass

    class XJob():
        def __init__(self, ctx):
            pass

else:
    g_ImplementationHelper = unohelper.ImplementationHelper()

class MendeleyDesktopAPI(unohelper.Base, XJob):
    def __init__(self, ctx):
        self.closed = 0
        self.ctx = ctx # component context
        self._client = MendeleyHttpClient()
        
        self._formattedCitationsResponse = MendeleyHttpClient.ResponseBody()

        self.citationClusters = []
        self.citationStyleUrl = ""
        self.formattedBibliography = []

        self._previousResultLength = 0

    def resetCitations(self):
        self.citationClusters = []

    def _citationClusterFromFieldCode(self, fieldCode):
        # remove ADDIN and CSL_CITATION from start
        pattern = re.compile("CSL_CITATION[ ]*({.*$)")
        match = pattern.search(fieldCode)

        if match == None:
            result = {"fieldCode" : fieldCode.decode('string_escape')}
        else:
            bareJson = match.group(1)
            citationCluster = json.loads(bareJson)
            result = {"citationCluster" : citationCluster}
        return result

    def addCitationCluster(self, fieldCode, footnoteIndex):
        self.citationClusters.append(self._citationClusterFromFieldCode(fieldCode))
        self.citationClusters[len(self.citationClusters)-1]["footnoteIndex"] = footnoteIndex

    def addFormattedCitation(self, formattedCitation):
        self.citationClusters[len(self.citationClusters)-1]["formattedText"] = formattedCitation
    
    def setCitationStyle(self, citationStyleUrl):
        self.citationStyleUrl = citationStyleUrl

    def getCitationStyleId(self):
        return self.citationStyleUrl

    def getCitationStylePresentationType(self):
        response = self._client.citationStylePresentationType()

        if response.status != 200:
            raise MendeleyHttpClient.UnexpectedResponse(response)

        return response.body.presentationType

    def formatCitationsAndBibliography(self):
        self._formattedCitationsResponse = \
            self._client.formattedCitationsAndBibliography_Interactive(
            self.citationStyleUrl, self.citationClusters).body

        return json.dumps(self._formattedCitationsResponse.__dict__)

    def getCitationCluster(self, index):
        return "ADDIN CSL_CITATION " + json.dumps(self._formattedCitationsResponse.citationClusters[int(index)]["citationCluster"])
#        return "citation cluster"

    def getFormattedCitation(self, index):
        return self._formattedCitationsResponse.citationClusters[int(index)]["formattedText"]

    def getFormattedBibliography(self):
        # a single string is interpreted as a file name
        if (type(self._formattedCitationsResponse.bibliography) == type(u"unicode string")
                or type(self._formattedCitationsResponse.bibliography) == type("string")):
            return self._formattedCitationsResponse.bibliography;
        else:
            return "<br/>".join(self._formattedCitationsResponse.bibliography)
        
    def getUserAccount(self):
        response = self._client.userAccount()
        
        if (response.status != 200):
            raise MendeleyHttpClient.UnexpectedResponse(response)

        return response.body.account

    def getUserUuid(self):
        response = self._client.userUuid()
        
        if (response.status != 200):
            raise MendeleyHttpClient.UnexpectedResponse(response)

        return response.body.uuid

    def citationStyle_choose_interactive(self, styleId):
        return self._client.citationStyle_choose_interactive(
            {"currentStyleUrl": styleId}).body.citationStyleUrl

    def citation_choose_interactive(self, hintText):
        response = self._client.citation_choose_interactive(
            {"citationEditorHint": hintText})
        try:
            assert(response.status == 200)
            fieldCode = self._fieldCodeFromCitationCluster(response.body.citationCluster)
        except:
            raise MendeleyHttpClient.UnexpectedResponse(response)

        return fieldCode
    
    def citation_edit_interactive(self, fieldCode, hintText):
        citationCluster = self._citationClusterFromFieldCode(fieldCode)
        citationCluster["citationEditorHint"] = hintText
        response = self._client.citation_edit_interactive(citationCluster)
        try:
            assert(response.status == 200)
            fieldCode = self._fieldCodeFromCitationCluster(response.body.citationCluster)
        except:
            raise MendeleyHttpClient.UnexpectedResponse(response)
        return fieldCode
    
    def setDisplayedText(self, displayedText):
        self.formattedText = displayedText

    def citation_update_interactive(self, fieldCode, formattedText):
        citationCluster = self._citationClusterFromFieldCode(fieldCode)
        citationCluster["formattedText"] = formattedText

        response = self._client.citation_update_interactive(citationCluster)
        try:
            assert(response.status == 200)
            fieldCode = self._fieldCodeFromCitationCluster(response.body.citationCluster)
        except:
            raise MendeleyHttpClient.UnexpectedResponse(response)
        return fieldCode

    def getFieldCodeFromUuid(self, documentUuid):
        response = self._client.testMethods_citationCluster_getFromUuid(
            {"documentUuid": documentUuid})
        try:
            assert(response.status == 200)
            fieldCode = self._fieldCodeFromCitationCluster(response.body.citationCluster)
        except:
            raise MendeleyHttpClient.UnexpectedResponse(response)
        return fieldCode

    def getDesktopSelectedStyleId(self):
        response = self._client.citationStyle_selected()

        if (response.status != 200):
            raise MendeleyHttpClient.UnexpectedResponse(response)

        return response.body.citationStyleId

    def _fieldCodeFromCitationCluster(self, citationCluster):
        if ("citationItems" in citationCluster):
            if (len(citationCluster["citationItems"]) == 0):
                return ""

        return "ADDIN CSL_CITATION " + json.dumps(citationCluster, sort_keys=True)

    def citation_undoManualFormat(self, fieldCode):
        citationCluster = self._citationClusterFromFieldCode(fieldCode)
        response = self._client.citation_undoManualFormat(citationCluster)
        try:
            assert(response.status == 200)
            fieldCode = self._fieldCodeFromCitationCluster(response.body.citationCluster)
        except:
            raise MendeleyHttpClient.UnexpectedResponse(response)
        return fieldCode

    def citations_merge(self, *fieldCodes):
        clusters = []

        for fieldCode in fieldCodes:
            clusters.append(self._citationClusterFromFieldCode(fieldCode))

        response = self._client.citations_merge({"citationClusters": clusters})
        try:
            assert(response.status == 200)
            mergedFieldCode = \
                self._fieldCodeFromCitationCluster(response.body.citationCluster)
        except:
            raise MendeleyHttpClient.UnexpectedResponse(response)
        
        return mergedFieldCode

    def wordProcessor_set(self, wordProcessor, version):
        response = self._client.wordProcessor_set(
            {
                "wordProcessor": wordProcessor,
                "version": version
            })

        try:
            assert(response.status == 200)
        except:
            raise MendeleyHttpClient.UnexpectedResponse(response)

        return ""

    def mendeleyDesktopInfo(self):
        response = self._client.mendeleyDesktopInfo()
        try:
            assert(response.status == 200)
        except:
            raise MendeleyHttpClient.UnexpectedResponse(response)

        result = {"processId": response.body.processId}
        return result

    def isMendeleyDesktopRunningStr(self):
        try:
            response = self._client.mendeleyDesktopInfo()
            return str(response.status == 200)
        except:
            return False

    # for testing
    def setNumberTest(self, number):
        self.number = number.decode('string_escape')
        return ""

    # for testing
    def getNumberTest(self):
        return str(self.number)

    # for testing
    def concatenateStringsTest(self, string1, string2):
        return str(string1) + str(string2)

    def previousSuccess(self):
        previousResponse = self._client.previousResponse
        
        return str(previousResponse.status == 200)

    def previousErrorMessage(self):
        previousResponse = self._client.previousResponse
        
        downloadInstructions = "Please download the latest version " + \
                "of Mendeley Desktop here: \n" + \
                "http://www.mendeley.com/download-mendeley-desktop"

        if (previousResponse.status == 406 or 
            previousResponse.status == 415):
            if (previousResponse.contentType.startswith(
                "application/vnd.mendeley.typeDeprecatedError")):
                # TODO: insert link to plugin download
                return "Deprecated type error. Please update this plugin to work with the " + \
                    "current version of Mendeley Desktop"
            else:
                return "Unknown type error. " + downloadInstructions

        if (previousResponse.status == 404):
            return "Page not found. " + downloadInstructions

        if (previousResponse.status != 200):
            return "Unknown error\n" + json.dumps(previousResponse.__dict__)

        return ""

    def previousResultLength(self):
        return self._previousResultLength

    def previousResponse(self):
        return json.dumps(self.previousResponse.__dict__)

    def execute(self, args):
        functionName = str(args[0].Value)
        statement = 'self.' + functionName + '('
        for arg in range(1, len(args)):
            statement += '"'
            data = codecs.getencoder('unicode_escape')(args[arg].Value)[0]
            if python3:
                data = data.decode('utf-8')
            statement += data.replace('"', '\\"')
            statement += '"'
            if arg < len(args) - 1:
                statement += ', '
        statement += ')'

        if hasattr(self, functionName):
            try:
                result = eval(statement)
                self._previousResultLength = len(unicode(result))
                return result
            except MendeleyHttpClient.UnexpectedResponse:
                return ""
        else:
            raise Exception("ERROR: Function " + functionName + " doesn't exist")

if not fakeUnoHelper:
    g_ImplementationHelper.addImplementation(MendeleyDesktopAPI,
        "org.openoffice.pyuno.MendeleyDesktopAPI",
        ("com.sun.star.task.MendeleyDesktopAPI",),)
